import es
import os
import sqlite3
import installlib
# import cmdlib
import gamethread
# import zipfile
import time
import re
from operator import truediv # I am not a real fan of __future__.division



update_check_enabled = True



info = es.AddonInfo()
info.basename = 'bansaver'
info.publicvar = info.basename + '_ver'
info.name = "BanSaver"
info.author = "Brainsucker"
info.description = "Saves NON-Permanent bans. So the bans will not expire after a server restart."
info.url = "http://addons.eventscripts.com/addons/view/bansaver"
info.version = "1.3.4"


class UpdateHandler(object):
    def __init__(self):
        self.installer = installlib.getInstaller('bansaver')
    
    def getLatestVersion(self):
        return self.installer.addon.currentversion
        
    def getInstalledVersion(self):
        return info['version']
    
    def makeUpdate(self, forceUpdate=False):
        logger.logInfo('Checking for new Version.', 2)
        updateStatus = self.installer.update(forceUpdate)
        if forceUpdate:
            logger.logInfo('Addon has been forced to update.', 1)
        if updateStatus == self.installer.STATUS_NO_INSTALL_INFO:
            msg = 'No Install Info has been found. Unable to check for updates.'
            logger.logDebug(msg, 1)
            es.dbgmsg(0, '[%s] %s' % (info['name'], msg))
            self.makeUpdate(True) # Force to update one time, to install the missing info correctly.
            
        elif updateStatus == self.installer.STATUS_NOT_APPROVED:
            msg = 'Addon is not approved. Unable to check for updates.'
            logger.logDebug(msg, 1)
            es.dbgmsg(0, '[%s] %s' % (info['name'], msg))
        
        elif updateStatus == self.installer.STATUS_SUCCESSFUL:
            logger.logInfo('New Version (v%s) has been installed successfully.' % self.getLatestVersion())
            es.dbgmsg(0, '[%s] Updated to new Version (v%s). Please reload the Addon or restart your Server.' % (info['name'], self.getLatestVersion()))
            
        elif updateStatus == self.installer.STATUS_ALREADY_DONE:
            msg = 'Addon is up to date.'
            logger.logDebug(msg, 1)
            es.dbgmsg(0, '[%s] %s' % (info['name'], msg))
            
            
        else:
            logger.logDebug('Updatecheck finished with status: %i' % updateStatus, 1)
            
uh = UpdateHandler()
    
    
addonpath = es.getAddonPath('bansaver')
# backup_basepath = os.path.join(addonpath, 'backups')

bantime_regex = re.compile(r'for ([\d\.]+) minutes', re.I)

class LogHandler(object):
    basepath = os.path.join(addonpath, 'logs')
    
    def __init__(self):
        self.loglvl = 0
        self.debuglvl = 0
        
    def log(self, text):
        self.filename = 'bansaver%s.log' % time.strftime('%y%m%d')
        
        yearpath = os.path.join(self.basepath, time.strftime('%Y'))
        monthpath = os.path.join(yearpath, time.strftime('%B'))
        daypath = os.path.join(monthpath, time.strftime('%d'))
        
        if not os.path.isdir(daypath):
            os.makedirs(daypath)
            
        p = os.path.join(daypath, self.filename)
        
        fileExists = os.path.isfile(p)
        
        f = open(p, 'a')
        
        if not fileExists:
            self.log('Logfile has been created.')
            
        f.write('[%s] %s\n' % (time.strftime('%X'), text))
        f.close()
        
    def logInfo(self, text, lvl=0):
        if self.loglvl >= lvl:
            self.log('[INFO] %s' % text)
        
    def logWarning(self, text):
        # Warnings will not be affected by the loglevel.
        self.log('[WARNING] %s' % text)
        
    def logDebug(self, text, lvl=0):
        if self.debuglvl >= lvl:
            self.log('[DEBUG] %s' % text)
        
logger = LogHandler()
# logger.loglvl = 3
# logger.debuglvl = 3

class SQLiteDB(object):
    dbpath = os.path.join(addonpath, 'banned.sql')
    
    def __init__(self):
        self.connection = sqlite3.connect(self.dbpath)
        self.cursor = self.connection.cursor()
        
        
        self.execute("CREATE TABLE IF NOT EXISTS banned_user (steamid TEXT UNIQUE, ip TEXT UNIQUE, expiredate REAL)")
        
        self.removeExpired()
        
        self.registerBans()
        
        self._autoCommit()
        
    def removeExpired(self):
        result = self.execute("DELETE FROM banned_user WHERE expiredate<=?", (time.time(), ))
        rc = result.rowcount
        logger.logInfo('Successfully removed %i entry(s) due expiry.' % rc)
        
    def registerBans(self):
        delay_time = 0 # Delay all bans a bit, when loading the addon. This will avoid a buffer overflow.
        result = self.execute("SELECT * FROM banned_user")
        countID = 0
        countIP = 0
        for id, ip, ed in result:
            delay_time += 0.1
            remaining_seconds = ed - time.time()
            remaining_minutes = truediv(remaining_seconds, 60)
            if id:
                gamethread.delayed(delay_time, es.server.queuecmd, ('banid %f %s' % (remaining_minutes, id)))
                countID += 1
                
            if ip:
                gamethread.delayed(delay_time, es.server.queuecmd, ('addip %f %s' % (remaining_minutes, ip)))
                countIP += 1
                
        logger.logInfo('Successfully restored %i ID and %i IP bans to the server.' % (countID, countIP))
                
    def addID(self, id, expiry):
        self.execute("INSERT OR REPLACE INTO banned_user(steamid, expiredate) VALUES(?, ?)", (id, expiry))
        logger.logInfo('Added ID (%s) to DB.' % id, 1)
        
    
    def addIP(self, ip, expiry):
        self.execute("INSERT OR REPLACE INTO banned_user(ip, expiredate) VALUES(?, ?)", (ip, expiry))
        logger.logInfo('Added IP (%s) to DB.' % ip, 1)
                
    def removeID(self, id):
        self.execute("DELETE FROM banned_user WHERE steamid=?", (id,))
        logger.logInfo('Removed ID (%s) from DB.' % id, 1)
        
    def removeIP(self, ip):
        self.execute("DELETE FROM banned_user WHERE ip=?", (ip,))
        logger.logInfo('Removed IP (%s) from DB.' % ip, 1)
        
    def execute(self, query, args=None):
        if args:
            return self.cursor.execute(query, args)
        else:
            return self.cursor.execute(query)
            
    def commit(self):
        self.connection.commit()
        logger.logInfo('DB has been committed.', 3)
        
    def close(self):
        self.cursor.close()
        logger.logInfo('DB has been closed.', 2)
        gamethread.cancelDelayed('bansaver_autocommit')
        
    def _autoCommit(self):
        gamethread.delayedname(60, 'bansaver_autocommit', self._autoCommit)
        self.commit()
        
db = SQLiteDB()


# def clear_db(args):
    # if args[0].lower() == 'help':
        # es.dbgmsg(0, 'Argument Syntax: <command> [backup]')
        # es.dbgmsg(0, '1. Argument: Defines if you want the script to create a backup of all bans.')
        # return
        
        
    # if args[0].lower() == 'backup':
        # bfp = os.path.join(backup_basepath, os.sep.join(time.strftime('%Y %B %d').split()))
        # if not os.path.isdir(bfp):
            # os.makedirs(bfp)
        
        # bfname = time.strftime('%H%M%S') + '.zip'
        # full_bfpath = os.path.join(bfp, bfname)
        # zfn = '%s.zip' % bfname
        # zf = zipfile.ZipFile(full_bfpath, mode='w')
        # zf.write(dbpath)
        # zf.close()
        
        
    # sql_cur.execute("DELETE FROM banned_user")
    # sql_db.commit()
    

def load():
    
    es.ServerVar(info["publicvar"], info["version"]).makepublic()
    
    uh.makeUpdate()
    
    
    # cmdlib.registerServerCommand('bansaver_cleardb', clear_db, 'Purges whole bansaver database. (USE WITH CAUTION!)')
    
def unload():
    db.commit()
    db.close()
    # cmdlib.unregisterServerCommand('bansaver_cleardb')
    

def es_map_start(ev):
    if update_check_enabled:
        uh.makeUpdate()


def server_addban(ev):
    steamid = ev["networkid"]
    ip = ev['ip']
    dur = ev["duration"]
    if dur == 'permanently':
    
        if steamid:
            db.removeID(steamid)
    
        if ip:
            db.removeIP(ip)
            
            
    else:
        m = bantime_regex.match(dur)
        if m:
            bantime = float(m.group(1))
            ed = time.time() + (bantime * 60)
            if steamid:
                db.addID(steamid, ed)
                    
            if ip:
                db.addIP(ip, ed)
                
                
    
            
def server_removeban(ev):
    steamid = ev["networkid"]
    ip = ev['ip']
    if steamid:
        db.removeID(steamid)
    if ip:
        db.removeIP(ip)
        